<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xs="http://www.w3.org/2001/XMLSchema" 
	xmlns:fn="http://www.w3.org/2005/xpath-functions"
	xmlns:uml="href://org.omg/UML/1.3">

	<xsl:output method="text" version="1.0" encoding="UTF-8"/>
	
	<!-- On filtre uniquement le package Logical View, là où se trouve notre diagramme de classe et de séquence-->
	<xsl:template match="//uml:Model[@name='Logical View']">
		<xsl:text>
		</xsl:text>
		<!-- On affiche chaque classe et sa composition -->
		<xsl:for-each select="uml:Namespace.ownedElement/uml:Class">
			<xsl:text>Classe : </xsl:text><xsl:value-of select="@visibility"/><xsl:text> </xsl:text>
				<xsl:value-of select="@name"/>
					<xsl:text>
					</xsl:text>
					
			<!-- On comprendra comme "attributs" les attributs de classe, bien qu'il n'y en ai aucun de base dans le diagramme de classe -->
			<xsl:if test="uml:Classifier.feature/uml:Attribute">
				<xsl:text>Les attributs sont : 
				</xsl:text>
				<xsl:for-each select="uml:Classifier.feature/uml:Attribute">
					<xsl:variable name="link_type"><xsl:value-of select="@type"/></xsl:variable>
					<xsl:text>    - </xsl:text><xsl:value-of select="@visibility"/><xsl:text> </xsl:text>
						<xsl:value-of select="//uml:Class[@xmi.id=$link_type]/@name"/><xsl:text> </xsl:text>
						<xsl:value-of select="@name"/>
						<xsl:text>
						</xsl:text>
				</xsl:for-each>
			</xsl:if>
			
			<!-- On listera aussi les opérations afin de mieux décrire la composition de la classe -->
			<xsl:if test="uml:Classifier.feature/uml:Operation">
				<xsl:text>Les opérations sont : 
				</xsl:text>
				<xsl:for-each select="uml:Classifier.feature/uml:Operation">
					<xsl:variable name="link_type"><xsl:value-of select="@type"/></xsl:variable>
					<xsl:variable name="link_return"><xsl:value-of select="uml:BehavioralFeature.parameter/uml:Parameter[@kind='return']/@type"/></xsl:variable>
					<xsl:text>    - </xsl:text><xsl:value-of select="@visibility"/><xsl:text> </xsl:text>
						<xsl:value-of select="//uml:Class[@xmi.id=$link_return]/@name"/><xsl:text> </xsl:text>
						<xsl:value-of select="//uml:Class[@xmi.id=$link_type]/@name"/><xsl:text> </xsl:text>
						<xsl:value-of select="@name"/>
						<xsl:text>
						</xsl:text>
				</xsl:for-each>
			</xsl:if>
			
			<!--On listera les associations en décrivant le type de la classe pointé ainsi que le nom de l'attribut décrivant la référence-->
			<xsl:text>Cette classe est relié à 
			</xsl:text>
			<!-- identifier interne de la class surlaquelle on travaille -->
			<xsl:variable name="class_id"><xsl:value-of select="@xmi.id"/></xsl:variable>
			<!-- on parcourt toutes les associations dont le départ est notre classe-->
			<xsl:for-each select="//uml:Association.connection[uml:AssociationEnd[1]/@type=$class_id]">
				<!-- on identifie le type de l'arrivée de l'association -->
				<xsl:variable name="endpoint"><xsl:value-of select="uml:AssociationEnd[2]/@type"/></xsl:variable>
				<!-- on affiche la classe correspondante en affichant le type et le nom de la classe d'arrivée-->
				<xsl:text>    - type </xsl:text><xsl:value-of select="//uml:Class[@xmi.id=$endpoint]/@name"/>
					<xsl:text> nom </xsl:text><xsl:value-of select="uml:AssociationEnd[2]/@name"/>
					
					<!-- affichage de la cardinalite si celle-ci est disponible -->
					<xsl:if test="uml:AssociationEnd[2]/uml:AssociationEnd.multiplicity">
						<xsl:text>, cardinalite : [</xsl:text>
							<xsl:choose>
								<xsl:when test="uml:AssociationEnd[2]/uml:AssociationEnd.multiplicity/uml:Multiplicity/uml:Multiplicity.range/uml:MultiplicityRange/@lower=-1">
								<xsl:text>*</xsl:text></xsl:when>
								<xsl:otherwise><xsl:value-of select="uml:AssociationEnd[2]/uml:AssociationEnd.multiplicity/uml:Multiplicity/uml:Multiplicity.range/uml:MultiplicityRange/@lower"/></xsl:otherwise>
							</xsl:choose>
							<xsl:text> ;</xsl:text>
							<xsl:choose>
								<xsl:when test="uml:AssociationEnd[2]/uml:AssociationEnd.multiplicity/uml:Multiplicity/uml:Multiplicity.range/uml:MultiplicityRange/@upper=-1">
								<xsl:text>*</xsl:text></xsl:when>
								<xsl:otherwise><xsl:value-of select="uml:AssociationEnd[2]/uml:AssociationEnd.multiplicity/uml:Multiplicity/uml:Multiplicity.range/uml:MultiplicityRange/@upper"/></xsl:otherwise>
							</xsl:choose>
							<xsl:text>]</xsl:text>
					</xsl:if>
					<xsl:text>
					</xsl:text>
			</xsl:for-each>
			
			<!-- On listera les messaegs envoyés par les instances de cette classes aux autres classes en énonçant les destinaitaires-->
			<xsl:text>Les messages envoyés par cette classe sont : 
			</xsl:text>
			
			<!-- identification du classifer qui correspond a notre classe -->
			<xsl:variable name="classifier_id">
				<xsl:value-of select="//uml:ClassifierRole[@base=$class_id]/@xmi.id"/>
			</xsl:variable>
			
			<!-- recherche de tous les messages envoyées -->
			<xsl:for-each select="//uml:Message[@sender=$classifier_id]">
			
				<!-- identification du receiver -->
				<xsl:variable name="message_receiver">
					<xsl:value-of select="@receiver"/>
				</xsl:variable>
				<xsl:variable name="receiver_base_class">
					<xsl:value-of select="//uml:ClassifierRole[@xmi.id=$message_receiver]/@base"/>
				</xsl:variable>
				<xsl:variable name="resolved_class_name">
					<xsl:value-of select="//uml:Class[@xmi.id=$receiver_base_class]/@name"/>
				</xsl:variable>
				
				<!-- affichage de la classe destinatrice et du message envoyé -->
				<xsl:text>    - message envoyé à </xsl:text><xsl:value-of select="$resolved_class_name"/>
					<xsl:text> : </xsl:text><xsl:value-of select="@name"/>
					<xsl:text>
					</xsl:text>
			</xsl:for-each>
			
		</xsl:for-each>
	</xsl:template>
	
</xsl:stylesheet>
